String.prototype.replaceAll = function(find, replace) {
  return this.replace(new RegExp(find, "g"), replace);
};
String.prototype.format = function(params) {
  var args = params;
  return this.replace(/{(\d+)}/g, function(match, number) { 
    return typeof args[number] != 'undefined'
      ? args[number]
      : match
    ;
  });
};

function removeLineWithPlaceholder(text, placeholder) {

  var ret = "";

  var lines = text.split(NEWLINE);

  for(var i = 0, len = lines.length; i < len; i++) {
    if (lines[i].indexOf(placeholder) === -1) {
      ret += lines[i];
      if (i !== len - 1) ret += NEWLINE;
    }

  }

  return ret;
}

var NEWLINE = process.platform === "win32" ? "\r\n" : "\n";
var TAB = "\t";

var PH_COLUMN_FOREIGN_KEY = "{contract_column_foreign_key}";
var PH_COLUMN_BUILD_HAS_MANY_URI = "{contract_build_has_many_uri}";

var PH_DATABASE_COLUMN_FOREIGN_KEY = "{database_column_foreign_key}";
var PH_DATABASE_TABLE_JOIN = "{database_table_join}";

var PH_PROVIDER_PATH_JOIN = "{provider_path_join}";
var PH_PROVIDER_MATCHER_ADD_URI = "{provider_matcher_add_uri}";
var PH_PROVIDER_GET_TYPE = "{provider_get_type}";
var PH_PROVIDER_QUERY = "{provider_query}";
var PH_PROVIDER_BUILD_SELECTION = "{provider_build_selection}";

var PH_QUERIES_PROJECTION = "{queries_projection}";
var PH_QUERIES_INDEX = "{queries_index}";


exports.preExecute = function(file) {
	console.log("start one2many");
}
exports.postExecute = function(file) {
	console.log("end one2many");
}

exports.replace = function(data, content) {

	var ret = content;
	return ret;

};

exports.replaceInTableLoop = function(tables, table, content) {

  var ret = content;

  ret = replaceContractColumnForeignKey(tables, table, ret);
  ret = replaceContractBuildHasManyUri(tables, table, ret);

  ret = replaceDatabaseColumnForeignKey(tables, table, ret);
  ret = replaceDatabaseTableJoin(tables, table, ret);

  ret = replaceProviderPathJoin(tables, table, ret);
  ret = replaceProviderMatcherAddUri(tables, table, ret);
  ret = replaceProviderGetType(tables, table, ret);
  ret = replaceProviderQuery(tables, table, ret);
  ret = replaceProviderSelection(tables, table, ret);

  ret = replaceQueriesProjection(tables, table, ret);
  ret = replaceQueriesIndex(tables, table, ret);

  return ret;

};

exports.replaceInColumnLoop = function(tables, table, column, content) {

  var ret = content;
  return ret;

};


function replaceContractColumnForeignKey(tables, table, content) {

  if (table.belongs_to !== '') {

    var template = "String {0}_ID = \"{1}_id\";";
    var params = [
      table.belongs_to_table.name_sing_up,
      table.name_sing + "_" + table.belongs_to_table.name_sing
    ];

    return content.replaceAll(PH_COLUMN_FOREIGN_KEY, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_COLUMN_FOREIGN_KEY);
  }

}

function replaceContractBuildHasManyUri(tables, table, content) {

  if (table.has_many !== '') {

    var template = "";

    template += "public static Uri build{0}Uri(String id) {" + NEWLINE;
    template += TAB + TAB + TAB + "return CONTENT_URI.buildUpon().appendPath(id).appendPath(PATH_{1}).build();" + NEWLINE;
    template += TAB + TAB + "}";

    var params = [
      table.has_many_table.name_cap,
      table.has_many_table.name_up
    ];

    return content.replaceAll(PH_COLUMN_BUILD_HAS_MANY_URI, template.format(params));
  
  } else {
    return removeLineWithPlaceholder(content, PH_COLUMN_BUILD_HAS_MANY_URI);
  }

}

function replaceDatabaseColumnForeignKey(tables, table, content) {

  if (table.belongs_to !== '') {

    var template = "+ {0}Columns.{1}_ID + \" INTEGER NOT NULL\"";

    var params = [
      table.name_cap,
      table.belongs_to_table.name_sing_up
    ];

    return content.replaceAll(PH_DATABASE_COLUMN_FOREIGN_KEY, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_DATABASE_COLUMN_FOREIGN_KEY);
  }

}

function replaceDatabaseTableJoin(tables, table, content) {

  if (table.has_many !== '') {

    var template = "String {0}_JOIN_{1} = \"{2} JOIN {3} ON {2}._id = {3}.{4}\";";

    var params = [
      table.name_up,
      table.has_many_table.name_up,
      table.name,
      table.has_many_table.name,
      table.has_many_table.name_sing + "_" + table.name_sing + "_id"
    ];

    return content.replaceAll(PH_DATABASE_TABLE_JOIN, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_DATABASE_TABLE_JOIN);
  }

}

function replaceProviderPathJoin(tables, table, content) {

  if (table.has_many !== '') {

    var template = "private static final int {0}_ID_{1} = {2};";

    var params = [
      table.name_up,
      table.has_many_table.name_up,
      "{=(params.index + 1) * 100 + 2=}"
    ];

    return content.replaceAll(PH_PROVIDER_PATH_JOIN, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_PROVIDER_PATH_JOIN);
  }

}

function replaceProviderMatcherAddUri(tables, table, content) {

  if (table.has_many !== '') {

    var template = "matcher.addURI(authority, \"{0}/*/{1}\", {2}_ID_{3});";

    var params = [
      table.name,
      table.has_many_table.name,
      table.name_up,
      table.has_many_table.name_up
    ];

    return content.replaceAll(PH_PROVIDER_MATCHER_ADD_URI, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_PROVIDER_MATCHER_ADD_URI);
  }

}

function replaceProviderGetType(tables, table, content) {

  if (table.has_many !== '') {

    var template = "";

    template += "case {0}_ID_{1}:" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "return {2}.CONTENT_TYPE;"

    var params = [
      table.name_up,
      table.has_many_table.name_up,
      table.has_many_table.name_cap
    ];

    return content.replaceAll(PH_PROVIDER_GET_TYPE, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_PROVIDER_GET_TYPE);
  }

}

function replaceProviderQuery(tables, table, content) {

  if (table.has_many !== '') {

    var template = "";

    template += "case {0}_ID_{1}: {" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "final SelectionBuilder builder = buildSimpleSelection(uri);" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "Cursor c = builder.where(selection, selectionArgs).query(db, projection, sortOrder);" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "c.setNotificationUri(getContext().getContentResolver(), {2}.CONTENT_URI);" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "return c;" + NEWLINE;
    template += TAB + TAB + TAB + "}" + NEWLINE;

    var params = [
      table.name_up,
      table.has_many_table.name_up,
      table.has_many_table.name_cap
    ];

    return content.replaceAll(PH_PROVIDER_QUERY, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_PROVIDER_QUERY);
  }

}

function replaceProviderSelection(tables, table, content) {

  if (table.has_many !== '') {

    var template = "";

    template += "case {0}_ID_{1}: {" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "final String id = {2}.get{3}Id(uri);" + NEWLINE;
    template += TAB + TAB + TAB + TAB + "return builder.table(Tables.{0}_JOIN_{1})" + NEWLINE;
    template += TAB + TAB + TAB + TAB + TAB + ".where(Tables.{0} + \".\" + {2}._ID + \"=?\", id);" + NEWLINE;
    template += TAB + TAB + TAB + "}" + NEWLINE;

    var params = [
      table.name_up,
      table.has_many_table.name_up,
      table.name_cap,
      table.name_sing_cap,
      table.name,

    ];

    return content.replaceAll(PH_PROVIDER_BUILD_SELECTION, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_PROVIDER_BUILD_SELECTION);
  }

}

function replaceQueriesProjection(tables, table, content) {

  if (table.belongs_to !== '') {


    var template = "{app_cap}Contract.{0}.{1}_ID";

    var params = [
      table.name_cap,
      table.belongs_to_table.name_sing_up
    ];

    return content.replaceAll(PH_QUERIES_PROJECTION, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_QUERIES_PROJECTION);
  }

}

function replaceQueriesIndex(tables, table, content) {

  if (table.belongs_to !== '') {

    var template = "int {0}_ID = {1};";

    var index = table.columns.length + 1;

    var params = [
      table.belongs_to_table.name_sing_up,
      index
    ];

    return content.replaceAll(PH_QUERIES_INDEX, template.format(params));

  } else {
    return removeLineWithPlaceholder(content, PH_QUERIES_INDEX);
  }

}
