var fs = require('fs'),
    inflection = require("inflection");


String.prototype.up = function() {
  return this.toUpperCase();
};
String.prototype.low = function() {
  return this.toLowerCase();
};
String.prototype.cap = function() {
  return this.charAt(0).up() + this.slice(1);
};
String.prototype.sing = function() {
  return inflection.singularize(this) || this;
}
String.prototype.plural = function() {
  return inflection.pluralize(this) || this;
}
String.prototype.replaceAll = function(find, replace) {
  return this.replace(new RegExp(find, "g"), replace);
};


var Table = function(table, one2many) {
  this.name = table.name.low();
  this.name_up = table.name.up();
  this.name_cap = table.name.cap();

  this.name_sing = table.name.sing();
  this.name_sing_up = table.name.sing().up();
  this.name_sing_cap = table.name.sing().cap();

  if (typeof table.has_many === 'undefined') this.has_many = '';
  else this.has_many = table.has_many;

  if (this.has_many !== '' && one2many) {
    this.has_many_table = new Table(getTableByName(this.has_many), false);
  }

  if (typeof table.belongs_to === 'undefined') this.belongs_to = '';
  else this.belongs_to = table.belongs_to;

  if (this.belongs_to !== '' && one2many) {
    this.belongs_to_table = new Table(getTableByName(this.belongs_to.plural()), false);
    if (typeof this.belongs_to_table === 'undefined') this.belongs_to_table = new Table(getTableByName(this.belongs_to), false);
  }

  this.columns = table.columns;
}

var Column = function(column, last) {
  this.name = column.name.low();
  this.name_up = column.name.up();
  this.name_cap = column.name.cap();

  if (typeof column.type === 'undefined') this.type = "TEXT";
  else this.type = column.type.up();

  if (typeof column.default === 'undefined') this.default = "";
  else this.default = " DEFAULT " + column.default;

  if (typeof column.primary_key === 'undefined') this.primary_key = "";
  else this.primary_key = column.primary_key ? " PRIMARY KEY" : "";

  if (typeof column.autoincrement === 'undefined') this.autoincrement = "";
  else this.autoincrement = column.autoincrement ? " AUTOINCREMENT" : "";

  if (typeof column.not_null === 'undefined') this.not_null = "";
  else this.not_null = column.not_null ? " NOT NULL" : "";

  this.comma = last ? "" : ",";
}

function getTableByName(name) {
  for (t = 0, t_len = _tables.length; t < t_len; t++) {
    if (_tables[t].name === name) return _tables[t];
  }
}


var NEWLINE = process.platform === "win32" ? "\r\n" : "\n";
var DIR_TEMPLATES = __dirname + "/templates/";
var DIR_CUSTOM = __dirname + "/custom/";

var DIR_PROVIDER = "provider";
var DIR_UTIL = "util";

var PH_PACKAGE = "{package}";
var PH_APP = "{app}";
var PH_APP_LOW = "{app_low}";
var PH_APP_CAP = "{app_cap}";
var PH_DATABASE = "{database}";

var PH_TABLE = "{table}";
var PH_TABLE_UP = "{table_up}";
var PH_TABLE_CAP = "{table_cap}";
var PH_TABLE_SING = "{table_sing}";
var PH_TABLE_SING_UP = "{table_sing_up}";
var PH_TABLE_SING_CAP = "{table_sing_cap}";

var PH_COLUMN = "{column}";
var PH_COLUMN_UP = "{column_up}";
var PH_COLUMN_CAP = "{column_cap}";
var PH_COLUMN_TYPE = "{column_type}";
var PH_COLUMN_DEFAULT = "{column_default}";
var PH_COLUMN_PRIMARY_KEY = "{column_primary_key}";
var PH_COLUMN_AUTOINCREMENT = "{column_autoincrement}";
var PH_COLUMN_NOT_NULL = "{column_not_null}";
var PH_COLUMN_COMMA = "{column_comma}";

var PH_LOOP_TABLES = "{foreach table}";
var PH_LOOP_TABLES_END = "{end table}";
var PH_LOOP_COLUMNS = "{foreach column}";
var PH_LOOP_COLUMNS_END = "{end column}";

var PH_EVAL_START = "{=";
var PH_EVAL_END = "=}";


String.prototype.replacePlaceholders = function(table, column) {
  
  var ret = this;

  ret = ret.replaceAll(PH_PACKAGE, _package);
  ret = ret.replaceAll(PH_APP, _app);
  ret = ret.replaceAll(PH_APP_LOW, _app.low());
  ret = ret.replaceAll(PH_APP_CAP, _app.cap());
  ret = ret.replaceAll(PH_DATABASE, _database);

  if (typeof table !== 'undefined') {
    ret = ret.replaceAll(PH_TABLE, table.name);
    ret = ret.replaceAll(PH_TABLE_UP, table.name_up);
    ret = ret.replaceAll(PH_TABLE_CAP, table.name_cap);
    ret = ret.replaceAll(PH_TABLE_SING, table.name_sing);
    ret = ret.replaceAll(PH_TABLE_SING_UP, table.name_sing_up);
    ret = ret.replaceAll(PH_TABLE_SING_CAP, table.name_sing_cap);
  }

  if (typeof column !== 'undefined') {
    ret = ret.replaceAll(PH_COLUMN, column.name);
    ret = ret.replaceAll(PH_COLUMN_UP, column.name_up);
    ret = ret.replaceAll(PH_COLUMN_CAP, column.name_cap);
    ret = ret.replaceAll(PH_COLUMN_TYPE, column.type);
    ret = ret.replaceAll(PH_COLUMN_DEFAULT, column.default);
    ret = ret.replaceAll(PH_COLUMN_PRIMARY_KEY, column.primary_key);
    ret = ret.replaceAll(PH_COLUMN_AUTOINCREMENT, column.autoincrement);
    ret = ret.replaceAll(PH_COLUMN_NOT_NULL, column.not_null);
    ret = ret.replaceAll(PH_COLUMN_COMMA, column.comma);
  }

  return ret;

}

String.prototype.replaceEvals = function(params) {
  
  var text = this;
  var ret = "";
  var start_outer = 0;
  var end_outer = 0;
  var start_inner = 0;
  var end_inner = 0;

  var lines = text.split(NEWLINE);

  for (var l = 0, len_l = lines.length; l < len_l; l++) {
    var line = lines[l];

    start_outer = line.indexOf(PH_EVAL_START);

    if (start_outer >= 0) {
      
      start_inner = start_outer + PH_EVAL_START.length;
      end_inner = line.indexOf(PH_EVAL_END, start_inner);
      end_outer = end_inner + PH_EVAL_END.length;

      var evalString = line.substring(start_inner, end_inner);
      var result = eval(evalString);

      lines[l] = line.substring(0, start_outer) + result + line.substring(end_outer, line.length);

    }

  }

  return lines.join(NEWLINE);

}


function generateDirectories(callback) {
  
  var done_provider = false;
  var done_util = false;

  fs.mkdir("./" + DIR_PROVIDER, function(err) {
    if (err) {
      if (err.errno != 47) console.log(err);
      console.log("Directory '" + DIR_PROVIDER + "' still existed.");
      done_provider = true;
      if (done_provider && done_util && typeof callback !== 'undefined') callback();
    } else {
      console.log("Created directory '" + DIR_PROVIDER + "'");
      done_provider = true;
      if (done_provider && done_util && typeof callback !== 'undefined') callback();
    }
  });

  fs.mkdir("./" + DIR_UTIL, function(err) {
    if (err) {
      if (err.errno != 47) console.log(err);
      console.log("Directory '" + DIR_UTIL + "' still existed.");
      done_util = true;
      if (done_provider && done_util && typeof callback !== 'undefined') callback();
    } else {
      console.log("Created directory '" + DIR_UTIL + "'");
      done_util = true;
      if (done_provider && done_util && typeof callback !== 'undefined') callback();
    }
  });

}

function generateFile(source, target) {

  fs.readFile(source, function(err, data) {

    var content = data.toString().replacePlaceholders();
    content = replaceTableLoops(content);

    writeFile(target, content);

    console.log("Generated file '" + target + "'");

  });
  
}

function writeFile(filename, content) {

  fs.writeFile("./" + filename, content, function(err) {
    if (err) {
      console.log(err);
      return false;
    } else {
      return true;
    }
  });

}


function replaceTableLoops(text) {
  
  var ret = "";
  var lines = text.split(NEWLINE);

  var table_loop_text = "";

  for (var l = 0, len_l = lines.length; l < len_l; l++) {

    // table loop found
    if (lines[l].indexOf(PH_LOOP_TABLES) >= 0) {

      table_loop_text = "";

      while (lines[++l].indexOf(PH_LOOP_TABLES_END) == -1) {
        table_loop_text += lines[l] + NEWLINE;
      }

      for (var t = 0, t_len = _tables.length; t < t_len; t++) {
        var table = new Table(_tables[t], true);

        var table_loop = replaceColumnLoops(table_loop_text, table);

        for (var i = 0, i_len = _customTemplates.length; i < i_len; i++) {
          table_loop = require(_customTemplates[i]).replaceInTableLoop(_tables, table, table_loop);
        }
        
        table_loop = table_loop.replacePlaceholders(table);
        table_loop = table_loop.replaceEvals({ "index": t });

        ret += table_loop;

      }

    } else {
      ret += lines[l] + NEWLINE;
    }

  }

  return ret;

}

function replaceColumnLoops(text, table) {

  if (text.indexOf(PH_LOOP_COLUMNS) == -1) return text;
  
  var ret = "";
  var lines = text.split(NEWLINE);

  var column_loop_text = "";

  for (var l = 0, len_l = lines.length; l < len_l; l++) {

    // column loop found
    if (lines[l].indexOf(PH_LOOP_COLUMNS) >= 0) {

      column_loop_text = "";

      while (lines[++l].indexOf(PH_LOOP_COLUMNS_END) == -1) {
        column_loop_text += lines[l] + NEWLINE;
      }

      for (var c = 0, c_len = table.columns.length; c < c_len; c++) {
        var column = new Column(table.columns[c], c == c_len-1 && table.belongs_to === '');
        ret += column_loop_text.replacePlaceholders(table, column).replaceEvals({ "index": c });
      }

    } else {
      ret += lines[l] + NEWLINE;
    }

  }

  return ret;

}



var _version = "",
    _app = "",
    _database = "",
    _package = "",
    _tables = [],
    _input = {},
    _customTemplates = [];

exports.run = function() {
  fs.readFile('./scaffoldroid.json', function (err, data) {

    console.log("");

    if (err && err.errno == 34) {
      console.log("scaffoldroid.json not found in current directory!");
      return;
    }
    
    try {
      _input = JSON.parse(data);
    } catch(e) {
      console.log("Error parsing scaffoldroid.json: " + e);
      return;
    }

    // _version = _input.scaffoldroid;
    _app = _input.app;
    _database = _input.database.low();
    _package = _input.package;
    _tables = _input.tables;

    // get custom templates
    fs.readdir(DIR_CUSTOM, function(err, files) {
      
      if (!err) {
        for (var i = 0, len = files.length; i < len; i++) {
          _customTemplates.push(DIR_CUSTOM + files[i]);      
        }
      }

      generateDirectories(function() {
      
        generateFile(DIR_TEMPLATES + 'database', './' + DIR_PROVIDER + '/' + _app.cap() + 'Database.java');
        generateFile(DIR_TEMPLATES + 'contract', './' + DIR_PROVIDER + '/' + _app.cap() + 'Contract.java');
        generateFile(DIR_TEMPLATES + 'provider', './' + DIR_PROVIDER + '/' + _app.cap() + 'Provider.java');
        generateFile(DIR_TEMPLATES + 'queries', './' + DIR_PROVIDER + '/' + _app.cap() + 'Queries.java');

        generateFile(DIR_TEMPLATES + 'lists', './' + DIR_UTIL + '/Lists.java');
        generateFile(DIR_TEMPLATES + 'maps', './' + DIR_UTIL + '/Maps.java');
        generateFile(DIR_TEMPLATES + 'selectionbuilder', './' + DIR_UTIL + '/SelectionBuilder.java');

      });

    });

  });
}

exports.run();